"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = buildJs;

var _path = _interopRequireDefault(require("path"));

var _gulp = require("gulp");

var _gulpIf = _interopRequireDefault(require("gulp-if"));

var _gulpGzip = _interopRequireDefault(require("gulp-gzip"));

var _gulpRename = _interopRequireDefault(require("gulp-rename"));

var _gulpUglify = _interopRequireDefault(require("gulp-uglify"));

var _browserify = _interopRequireDefault(require("browserify"));

var _babelify = _interopRequireDefault(require("babelify"));

var _envify = _interopRequireDefault(require("envify"));

var _licensify = _interopRequireDefault(require("licensify"));

var _vinylBuffer = _interopRequireDefault(require("vinyl-buffer"));

var _vinylSourceStream = _interopRequireDefault(require("vinyl-source-stream"));

var _gulpUtilErrorHandler = _interopRequireDefault(require("@hidoo/gulp-util-error-handler"));

var _utilMergeBabelrc = _interopRequireDefault(require("@hidoo/util-merge-babelrc"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * task default options.
 * @type {Object}
 */
const DEFAULT_OPTIONS = {
  // task name (set displayName)
  name: 'build:js',
  // source path (required)
  src: null,
  // destination path (required)
  dest: null,
  // destination filename
  filename: 'main.js',
  // target browsers
  // + http://browserl.ist/?q=%3E+0.5%25+in+JP%2C+ie%3E%3D+10%2C+android+%3E%3D+4.4
  browsers: ['> 0.5% in JP', 'ie >= 10', 'android >= 4.4'],
  // use polyfill or not
  // + see options.useBuiltIns in @babel/preset-env
  useBuiltIns: 'usage',
  // babelrc path
  babelrc: _path.default.resolve(process.cwd(), '.babelrc.js'),
  // compress file or not
  compress: false,
  // out log or not
  verbose: false
};
/**
 * return javascript build task by browserify
 * @param {DEFAULT_OPTIONS} options options
 * @return {Function}
 *
 * @example
 * import {task} from 'gulp';
 * import buildJs from '@hidoo/gulp-task-build-js-browserify';
 *
 * task('js', buildJs({
 *   src: '/path/to/js/main.js',
 *   dest: '/path/to/dest'
 * }));
 */

function buildJs(options = {}) {
  const opts = _objectSpread({}, DEFAULT_OPTIONS, options),
        {
    browsers,
    useBuiltIns,
    babelrc,
    verbose
  } = opts,
        babelifyOptions = (0, _utilMergeBabelrc.default)(babelrc, {
    presets: [['@babel/preset-env', {
      targets: {
        browsers
      },
      useBuiltIns: useBuiltIns,
      debug: verbose
    }]]
  }, {
    verbose
  }); // define task


  const task = () => {
    const {
      filename,
      compress
    } = opts;
    const bundler = (0, _browserify.default)(opts.src).transform(_babelify.default, babelifyOptions).transform(_envify.default, {
      NODE_ENV: process.env.NODE_ENV
    }) // eslint-disable-line no-process-env
    .plugin(_licensify.default, {
      includePrivate: true
    }).bundle().on('error', error => {
      (0, _gulpUtilErrorHandler.default)(error);
      bundler.emit('end');
    });
    return bundler.pipe((0, _vinylSourceStream.default)(filename)).pipe((0, _vinylBuffer.default)()).pipe((0, _gulp.dest)(opts.dest)).pipe((0, _gulpIf.default)(compress, (0, _gulpRename.default)({
      suffix: '.min'
    }))).pipe((0, _gulpIf.default)(compress, (0, _gulpUglify.default)({
      output: {
        comments: 'some'
      }
    }))).pipe((0, _gulpIf.default)(compress, (0, _gulp.dest)(opts.dest))).pipe((0, _gulpIf.default)(compress, (0, _gulpGzip.default)({
      append: true
    }))).pipe((0, _gulpIf.default)(compress, (0, _gulp.dest)(opts.dest)));
  }; // add displayName (used as task name for gulp)


  if (typeof opts.name === 'string' && opts.name !== '') {
    task.displayName = opts.name;
  }

  return task;
}